import { Observable } from 'rxjs';
import { AwaitableObservable, ConcurrencyMode, Listener, Thunk } from './types';
export { toggleMap } from './toggleMap';
export { concat } from 'rxjs';
export { map, tap, scan } from 'rxjs/operators';
/**
 * Returns a random hex string, like a Git SHA. Not guaranteed to
 * be unique - just to within about 1 in 10,000.
 */
export declare const randomId: (length?: number) => string;
/**
 * Returns an Observable of the value, or result of the function call, after
 * the number of milliseconds given. After is lazy and cancelable! So nothing happens until .subscribe
 * is called explicitly (via subscribe) or implicitly (toPromise(), await).
 * For a delay of 0, the function is executed synchronously when .subscribe is called.
 * @returns An Observable of the object or thunk return value. It is 'thenable', so may also be awaited directly.
 * ```
 * // Examples:
 * // awaited Promise
 * await after(100, () => new Date())
 * // unawaited Promise
 * after(100, () => new Date()).toPromise()
 * // unresolving Promise
 * after(Infinity, () => new Date()).toPromise()
 * ```
 */
export declare function after<T>(ms: number, objOrFn?: T | Thunk<T>): AwaitableObservable<T>;
/** Executes the given function on the microtask queue.
 * The microtask queue flushes before the macrotask queue.
 * @returns A Promise for its return value
 * @see https://developer.mozilla.org/en-US/docs/Web/API/HTML_DOM_API/Microtask_guide
 * @see queueMicrotask, Promise.resolve
 */
export declare function microq(fn: Function): any;
/** Executes the given function on the macrotask queue.
 * The macrotask queue flushes after the microstask queue.
 * @returns A Promise for its return value
 * @see https://developer.mozilla.org/en-US/docs/Web/API/HTML_DOM_API/Microtask_guide
 * @see setTimeout
 */
export declare function macroq(fn: Function): Promise<unknown>;
/** Returns a Promise for the point in time at which all existing queued microtasks (e.g. Promise.resolve()) have completed. */
export declare function microflush(): Promise<number>;
/** Returns a Promise for the point in time at which all existing queued macrotasks (e.g. setTimeout 0) have completed. */
export declare function macroflush(): Promise<number>;
/** Creates a derived Observable, running the listener in the given ConcurrencyMode
 * turning sync errors into Observable error notifications
 */
export declare function combineWithConcurrency<T, U>(o: Observable<T>, listener: Listener<T, U>, mode: ConcurrencyMode, individualPipes?: never[], individualEnder?: Observable<never>): Observable<unknown>;
